
#ifndef CLODDC_H
#define CLODDC_H

#include <string>
#include <sstream>
#include <cmath>
#include <cstring>
#include <iostream>
#include <ctime>







const int fighterStrength = 30;
const int bomberStrength = 70;
const int jaboStrength = 60;
const int artyStrength = 40;
const int redTankStrength = 15;
const int blueTankStrength = 20;
const int shipStrength = 20;
const int airfieldRebuildPeriod = 4;
const int portRebuildPeriod = 20;












static std::string assesResult = "NoString";
static bool resetPlayerLine = false;











template<class T, int inc = 50>
class seArray	{
		T** ap;
		int size;
		int quantity;
		int increment;
		void inflate();
	public:
		seArray() : ap(0), size(0), increment(inc), quantity(0)	{
		}
		~seArray();
		int add(T element);
		int add(T* element);
		int insert(int index, T element);
		int insert(int index, T* element);
		T operator[](int index);
		T* mod(int index);
		int getSize()	{
			return size;
		}
};


template<class T, int inc>
seArray<T, inc>::~seArray()	{
	for(int i = 0; i < size; i++)	{
		delete ap[i];
		ap[i] = 0;
	}
	delete []ap;
}

template<class T, int inc>
int seArray<T, inc>::add(T element)	{
	if(size >= quantity)
		inflate();
	ap[size++] = new T(element);
	return size - 1;
}

template<class T, int inc>
int seArray<T, inc>::add(T* element)	{
	if(size >= quantity)
		inflate();
	ap[size++] = element;
	return size - 1;
}

template<class T, int inc>
inline T seArray<T, inc>::operator[](int index)	{
	if(index < 0)	{
		return *ap[0];
	}
	if(index >= size)	{
		return *ap[size - 1];
	}
	return *ap[index];
}

template<class T, int inc>
inline T* seArray<T, inc>::mod(int index)	{
	if(index < 0)	{
		return ap[0];
	}
	if(index >= size)	{
		return ap[size - 1];
	}
	return ap[index];
}

template<class T, int inc>
void seArray<T, inc>::inflate()	{
	const int psz = sizeof(T*);
	T** st = new T*[quantity + increment];
	memset(st, 0, (quantity + increment) * psz);
	memcpy(st, ap, quantity * psz);
	quantity += increment;
	delete []ap;
	ap = st;
}

template<class T, int inc>
int seArray<T, inc>::insert(int index, T element)	{
	if((index < 0) || (index > (size - 1)))	{
		return 0;
	}
	const int psz = sizeof(T*);
	T** st = new T*[quantity + 1];
	memset(st, 0, (quantity + 1) * psz);
	for(int i = 0; i <= index; i++)
		st[i] = ap[i];
	st[index + 1] = new T(element);
	for(int i = index + 2; i <= size; i++)
		st[i] = ap[i - 1];
	delete []ap;
	ap = st;
	quantity++;
	size++;
	return index + 1;
}

template<class T, int inc>
int seArray<T, inc>::insert(int index, T* element)	{
	if((index < 0) || (index > (size - 1)))	{
		return 0;
	}
	const int psz = sizeof(T*);
	T** st = new T*[quantity + 1];
	memset(st, 0, (quantity + 1) * psz);
	for(int i = 0; i <= index; i++)
		st[i] = ap[i];
	st[index + 1] = element;
	for(int i = index + 2; i <= size; i++)
		st[i] = ap[i - 1];
	delete []ap;
	ap = st;
	quantity++;
	size++;
	return index + 1;
}




class position	{
	public:
		double positionX;
		double positionY;
		double altitude;
		position(double posX = 0, double posY = 0, double alt = 0) : positionX(posX), positionY(posY), altitude(alt)	{
		}
		position(const position& ppos) : positionX(ppos.positionX), positionY(ppos.positionY), altitude(ppos.altitude)	{
		}
		position operator=(const position& ppos)	{
			positionX = ppos.positionX;
			positionY = ppos.positionY;
			altitude = ppos.altitude;
			return *this;
		}
};








class wayPoint	{
	public:
		double numberWithS1;
		double numberWithS2;
		double numberWithS3;
		double numberWithS4;
		double positionX;
		double positionY;
		double speed;
		std::string allelse;
		bool passed;
		wayPoint(double nWs1 = 0, double nWs2 = 0, double nWs3 = 0, double nWs4 = 0, double posX = 0, double posY = 0, double spd = 0, std::string aelse = "NoString", bool pd = false) : numberWithS1(nWs1), numberWithS2(nWs2), numberWithS3(nWs3), numberWithS4(nWs4), positionX(posX), positionY(posY), speed(spd), allelse(aelse), passed(pd)	{
		}
		wayPoint(const wayPoint& pwp) : numberWithS1(pwp.numberWithS1), numberWithS2(pwp.numberWithS2), numberWithS3(pwp.numberWithS3), numberWithS4(pwp.numberWithS4), positionX(pwp.positionX), positionY(pwp.positionY), speed(pwp.speed), allelse(pwp.allelse), passed(pwp.passed)	{
		}
		wayPoint operator=(const wayPoint pwp)	{
			numberWithS1 = pwp.numberWithS1;
			numberWithS2 = pwp.numberWithS2;
			numberWithS3 = pwp.numberWithS3;
			numberWithS4 = pwp.numberWithS4;
			positionX = pwp.positionX;
			positionY = pwp.positionY;
			speed = pwp.speed;
			allelse = pwp.allelse;
			passed = pwp.passed;
			return *this;
		}
		bool isItSet()    {
			if(numberWithS1 == 0 && numberWithS2 == 0 && numberWithS3 == 0 && numberWithS4 == 0 && positionX == 0 && positionY == 0 && speed == 0)
				return false;
			else
				return true;
		}
		friend std::string grid(const position& pos);
		friend std::string grid(const wayPoint wp);
		friend std::string grid(const double coordX, const double coordY);
};






class path	{
	public:
		wayPoint wpoints[200];
		path() : wpoints()	{
		}
		~path()	{
		}
		int getLimit();
		wayPoint operator[](int index)	{
			if(getLimit() == -1)	{
				return wpoints[0];
			}
			if(index < 0 || index > getLimit())	{
				return wpoints[getLimit()];
			}
			return wpoints[index];
		}
};


class battleArea	{
		double leftVertical;
		double lowerHorizontal;
		double width;
		double heigth;
		double gridsize;
	public:
		battleArea(const std::string filename, double lV = 5000, double lH = 5000, double w = 360000, double h = 310000, double gs = 1000) : leftVertical(lV), lowerHorizontal(lH), width(w), heigth(h), gridsize(gs)	{
			getFromLine(filename);
		}
		std::string getBALine(std::string fnme);
		void getFromLine(const std::string finame);
		friend std::string grid(const double coordX, const double coordY, const std::string missFileName);
		friend std::string grid(position pos, const std::string missFileName);
		friend std::string grid(wayPoint wp, const std::string missFileName);
		double getLV() const	{
			return leftVertical;
		}
		double getLH() const	{
			return lowerHorizontal;
		}
		double getW() const	{
			return width;
		}
		double getH() const	{
			return heigth;
		}
};



wayPoint getWPCoordinates(const std::string pline);

position getCoordinates(const std::string pline);

double distance(const position& pos1, const position& pos2);

double distance(const wayPoint& wp1, const wayPoint& wp2);

double distance(const double coordX1, const double coordY1, const double coordX2, const double coordY2);

std::string grid(const double coordX, const double coordY, const std::string missFileName);

std::string grid(position pos, const std::string missFileName);

std::string grid(wayPoint wp, const std::string missFileName);

std::string addDots(std::string pstring);

std::string removeDots(std::string pstring);

std::string addUnderscores(std::string pstring);

std::string removeUnderscores(std::string pstring);

std::string addVerticals(std::string pstring);

std::string removeVerticals(std::string pstring);










class flightInfo	{
	public:
		int flightCount;
		std::string flight0[10];
		std::string flight1[10];
		std::string flight2[10];
		std::string flight3[10];
		bool flight0destroyed[10];
		bool flight1destroyed[10];
		bool flight2destroyed[10];
		bool flight3destroyed[10];
		std::string flight0skill[10];
		std::string flight1skill[10];
		std::string flight2skill[10];
		std::string flight3skill[10];
		std::string flight0aging[10];
		std::string flight1aging[10];
		std::string flight2aging[10];
		std::string flight3aging[10];
		std::string flight0skin[10];
		std::string flight1skin[10];
		std::string flight2skin[10];
		std::string flight3skin[10];
		std::string flight0serial[10];
		std::string flight1serial[10];
		std::string flight2serial[10];
		std::string flight3serial[10];
		flightInfo();
		int getflight0Limit();
		int getflight1Limit();
		int getflight2Limit();
		int getflight3Limit();
		bool flight0PlanesAvailable();
		bool flight1PlanesAvailable();
		bool flight2PlanesAvailable();
		bool flight3PlanesAvailable();
		int getArrayIndex(std::string pstring);
};



class detonator	{
	public:
		std::string detonatorLines[5];
		int detonatorCount;
		detonator();
		detonator operator=(const detonator& pdet)	{
			detonatorLines[0] = pdet.detonatorLines[0];
			detonatorLines[1] = pdet.detonatorLines[1];
			detonatorLines[2] = pdet.detonatorLines[2];
			detonatorLines[3] = pdet.detonatorLines[3];
			detonatorLines[4] = pdet.detonatorLines[4];
			detonatorCount = pdet.detonatorCount;
			return *this;
		}
		bool operator!=(const detonator& pdetr)	{
			if(detonatorLines[0] != pdetr.detonatorLines[0] || detonatorLines[1] != pdetr.detonatorLines[1] || detonatorLines[2] != pdetr.detonatorLines[2] || detonatorLines[3] != pdetr.detonatorLines[3] || detonatorLines[4] != pdetr.detonatorLines[4])
				return true;
			else
				return false;
		}
};



class belt	{
	public:
		std::string beltLines[10];
		int beltCount;
		belt();
		belt operator=(const belt& pbelt);
		bool operator!=(const belt& pbelt);
};



class planesData	{
	public:
		seArray<std::string, 5> Airgroups;
		seArray<flightInfo, 5> flights;
		seArray<std::string, 5> formations;
		seArray<std::string, 5> callsigns;
		seArray<std::string, 5> weapons;
		seArray<detonator, 5> detonators;
		seArray<belt, 5> belts;
		seArray<std::string, 5> type;
		seArray<std::string, 5> classid;
		seArray<double, 5> serviceCeiling;
		seArray<double, 5> actionRadius;
		seArray<int, 5> planesNo;
		seArray<double, 5> positionX;
		seArray<double, 5> positionY;
		seArray<std::string, 5> side;
		seArray<bool, 5> destroyed;
		planesData() : Airgroups(), flights(), formations(), callsigns(), weapons(), detonators(), belts(), type(), classid(), serviceCeiling(), actionRadius(), planesNo(), positionX(), positionY(), side(), destroyed()	{
		}
		bool check(const std::string pltype);
		void ask(std::string ptype);
		int availablePlanesNo(int index);
		void setAirgroupDestroyed(int index)	{
			int availablePlanes = availablePlanesNo(index);
			if(availablePlanes == 0)
				*(destroyed.mod(index)) = true;
			if(availablePlanes > 0)
				*(destroyed.mod(index)) = false;
			if(availablePlanes < 0)
				return;
		}
		void load(const std::string filename);
		void save();
		position makePosit(int index)	{
			if(index < 0)	{
				position result;
				return result;
			}
			if(Airgroups.getSize() < index)	{
				position result;
				return result;
			}
			position result = position(positionX[index], positionY[index]);
			return result;
		}
		bool anyAvailable(const std::string pclassid, const std::string pside);
};



class chiefsData	{
	public:
		seArray<std::string, 5> chiefname;
		seArray<std::string, 5> type;
		seArray<std::string, 5> classid;
		seArray<std::string, 5> countryid;
		seArray<std::string, 5> side;
		seArray<int, 5> vehiclesNo;
		seArray<std::string, 5> restOfit;
		seArray<path, 5> paths;
		seArray<bool, 5> reverse;
		seArray<bool, 5> justReversed;
		chiefsData() : chiefname(), type(), classid(), countryid(), side(), vehiclesNo(), restOfit(), paths(), reverse(), justReversed()	{
		}
		void load(const std::string filename);
		void save();
		bool anyAvailable(const std::string pclassid, const std::string pside);
};








class factory	{
	public:
		std::string classid;
		std::string side;
		int maxProductivity;
		std::string subassemblies[10];
		int subassemblyStockpile[10];
		std::string product[10];
		int productStockpile[10];
		std::string staticid[4];
		std::string bicycleid[4];
		std::string countryid[4];
		double positionX[4];
		double positionY[4];
		double thirdNumber[4];
		bool bicycleDestroyed[4];
		std::string grid;
		std::string name;
		factory();
		float bicycleNo();
		float productivity();
		int productsNo();
		int subassemblyNo();
		bool bicycleAffiliation(const position& ppos);
		position makePosit()	{
			position result(positionX[0], positionY[0], thirdNumber[0]);
			return result;
		}
};





class subcontractor	{
	public:
		std::string classid;
		std::string side;
		int maxProductivity;
		std::string mainfactoryName[10];
		std::string product[10];
		int stockpile[10];
		std::string staticid[4];
		std::string bicycleid[4];
		std::string countryid[4];
		double positionX[4];
		double positionY[4];
		double thirdNumber[4];
		bool bicycleDestroyed[4];
		std::string grid;
		std::string name;
		subcontractor();
		float bicycleNo();
		float productivity();
		int mainfactoryNo();
		bool bicycleAffiliation(const position& ppos);
		position makePosit()	{
			position result(positionX[0], positionY[0], thirdNumber[0]);
			return result;
		}
};



class factoriesData	{
	public:
		seArray<factory, 5> factories;
		seArray<subcontractor, 5> subcontractors;
		factoriesData() : factories(), subcontractors()	{
		}

		void load(const std::string filename);
		void save();
		friend class factory;
		friend class subcontractor;
		bool anyAvailable(const std::string pclassid, const std::string pside);
};



class facility	{
	public:
		std::string classid;
		std::string side;
		std::string staticid[4];
		std::string tractorid[4];
		std::string countryid[4];
		double positionX[4];
		double positionY[4];
		double thirdNumber[4];
		bool tractorDestroyed[4];
		long long ToTDestruction[4];
		std::string grid;
		std::string name;
		facility();
		virtual ~facility() = 0;
		bool destroyed();
		bool tractorAffiliation(const position& ppos);
		position makePosit()	{
			position result(positionX[0], positionY[0], thirdNumber[0]);
			return result;
		}
};

class airfield : public facility	{
	public:
		airfield() : facility()	{
			classid = "airfield";
		}
};

class port : public facility	{
	public:
		port() : facility()	{
			classid = "port";
		}
};



class facilitiesData	{
	public:
		seArray<airfield, 5> airfields;
		seArray<port, 5> ports;
		facilitiesData() : airfields(), ports()	{
		}
		void load(const std::string filename);
		void save();
		friend class airfield;
		friend class port;
		bool anyAvailable(const std::string pclassid, const std::string pside);
};



class staticObject	{
	public:
		std::string classid;
		std::string staticid;
		std::string type;
		std::string countryid;
		std::string side;
		bool destroyed;
		int tokenOfAttention;
		bool tokenOfLove;
		double positionX;
		double positionY;
		double terrainElevation;
		std::string allelse;
		std::string grid;
		std::string gridname;
		staticObject();
		virtual ~staticObject() = 0;
		void load(const std::string filename);
		void save();
		position makePosit()	{
			position result(positionX, positionY, terrainElevation);
			return result;
		}
};

class artillery : public staticObject	{
	public:
		artillery() : staticObject()	{
			classid = "artillery";
		}
};

class radar : public staticObject	{
	public:
		radar() : staticObject()	{
			classid = "radar";
		}
};





class staticData	{
	public:
		seArray<artillery, 10> guns;
		seArray<radar, 10> radars;
		staticData() : guns(), radars()	{
		}
		void load(const std::string filename);
		void save();
		bool anyAvailable(const std::string pclassid, const std::string pside);
};





class plane	{
	public:
		std::string classid;
		std::string groupname;
		std::string tacticalNo;
		std::string skill;
		std::string aging;
		std::string skin;
		std::string serial;
		std::string formation;
		std::string callsign;
		std::string weapons;
		detonator det;
		belt bt;
		std::string type;
		double actionRadius;
		double serviceCeiling;
		bool available;
		bool escorted;
		position wayPoints[20];
		double wayPointSpeed[20];
		std::string wayPointTypes[20];
		std::string wayPointActions[20];
		std::string briefingLines[20];
		int briefingNo;
		plane();
		virtual ~plane() = 0;
		int getWPLimit();
		int getBriefingLimit();
};


class fighter : public plane	{
	public:
		fighter() : plane()	{
			classid = "fighter";
		}
};


class bomber : public plane	{
	public:
		bomber() : plane()	{
			classid = "bomber";
		}
};


class jabo : public plane	{
	public:
		jabo() : plane()	{
			classid = "jabo";
		}
};




class vehicle	{
	public:
		std::string classid;
		std::string type;
		std::string chiefname;
		std::string countryid;
		std::string restOfit;
		int tokenOfAttention;
		bool tokenOfLove;
		int indexWithinGroup;
		path chiefPath;
		vehicle() : classid("NoString"), type("NoString"), chiefname("NoString"), countryid("NoString"), restOfit("NoString"), tokenOfAttention(0), tokenOfLove(false), indexWithinGroup(0), chiefPath()	{
		}
		position makePosit()	{
			position result;
			result.positionX = chiefPath.wpoints[0].positionX;
			result.positionY = chiefPath.wpoints[0].positionY;
			result.altitude = 0;
			return result;
		}
		virtual ~vehicle() = 0;
};


class vehiclect : public vehicle	{
	public:
		vehiclect() : vehicle()	{
			classid = "vehiclect";
		}
};


class tank : public vehicle	{
	public:
		tank() : vehicle()	{
			classid = "tank";
		}
};


class ship : public vehicle	{
	public:
		ship() : vehicle()	{
			classid = "ship";
		}
};


class train : public vehicle	{
	public:
		train() : vehicle()	{
			classid = "train";
		}
};



class factoryTarget	{
	public:
		std::string classid;
		position factoryPosit;
		std::string grid;
		std::string name;
		int tokenOfAttention;
		bool tokenOfLove;
		factoryTarget() : classid("NoString"), factoryPosit(), grid("NoString"), name("NoString"), tokenOfAttention(0), tokenOfLove(false)	{
		}
		std::string getBicycleID(std::string pFName);
};



class subcontractorTarget	{
	public:
		std::string classid;
		position subcontractorPosit;
		std::string grid;
		std::string name;
		int tokenOfAttention;
		bool tokenOfLove;
		subcontractorTarget() : classid("NoString"), subcontractorPosit(), grid("NoString"), name("NoString"), tokenOfAttention(0), tokenOfLove(false)	{
		}
		std::string getBicycleID(std::string pSName);
};



class airfieldTarget	{
	public:
		std::string classid;
		position airfieldPosit;
		std::string grid;
		std::string name;
		int tokenOfAttention;
		bool tokenOfLove;
		airfieldTarget() : classid("NoString"), airfieldPosit(), grid("NoString"), name("NoString"), tokenOfAttention(0), tokenOfLove(false)	{
		}
		std::string getTractorID(std::string pAFname);
};



class portTarget	{
	public:
		std::string classid;
		position portPosit;
		std::string grid;
		std::string name;
		int tokenOfAttention;
		bool tokenOfLove;
		portTarget() : classid("NoString"), portPosit(), grid("NoString"), name("NoString"), tokenOfAttention(0), tokenOfLove(false)	{
		}
		std::string getTractorID(std::string pPname);
};



class mainTables	{
	public:
		seArray<fighter, 3> fightersRed;
		seArray<bomber, 3> bombersRed;
		seArray<jabo, 3> jabosRed;
		seArray<fighter, 3> fightersBlue;
		seArray<bomber, 3> bombersBlue;
		seArray<jabo, 3> jabosBlue;
		seArray<vehiclect, 3> vehiclesctRed;
		seArray<tank, 3> tanksRed;
		seArray<train, 3> trainsRed;
		seArray<ship, 3> shipsRed;
		seArray<vehiclect, 3> vehiclesctBlue;
		seArray<tank, 3> tanksBlue;
		seArray<train, 3> trainsBlue;
		seArray<ship, 3> shipsBlue;
		seArray<factoryTarget, 3> factoryTargetsRed;
		seArray<subcontractorTarget, 3> subcontractorTargetsRed;
		seArray<factoryTarget, 3> factoryTargetsBlue;
		seArray<subcontractorTarget, 3> subcontractorTargetsBlue;
		seArray<airfieldTarget, 3> airfieldTargetsRed;
		seArray<portTarget, 3> portTargetsRed;
		seArray<airfieldTarget, 3> airfieldTargetsBlue;
		seArray<portTarget, 3> portTargetsBlue;
		seArray<artillery, 3> gunsRed;
		seArray<radar, 3> radarsRed;
		seArray<artillery, 3> gunsBlue;
		seArray<radar, 3> radarsBlue;
		mainTables(const std::string pfname) : fightersRed(), bombersRed(), jabosRed(), fightersBlue(), bombersBlue(), jabosBlue(), vehiclesctRed(), tanksRed(), trainsRed(), shipsRed(), vehiclesctBlue(), tanksBlue(), trainsBlue(), shipsBlue(), factoryTargetsRed(), subcontractorTargetsRed(), factoryTargetsBlue(), subcontractorTargetsBlue(), airfieldTargetsRed(), portTargetsRed(), airfieldTargetsBlue(), portTargetsBlue(), gunsRed(), radarsRed(), gunsBlue(), radarsBlue()	{
			loadUp(pfname);
		}
		void loadUp(const std::string fname);
};



class modeSetter	{
		bool mode;
	public:
		modeSetter() : mode(false)	{
			createFileIfNonExistant();
		}
		void load();
		void save();
		bool getMode() const	{
			return mode;
		}
		void setMode(const bool pmode)	{
			mode = pmode;
		}
		void createFileIfNonExistant();
};



class pattern	{
		std::string nameOfFile;
	public:
		seArray<std::string, 20> missionMis;
		seArray<int, 5> LineIndex;
		seArray<std::string> uniqueid;
		seArray<bool, 5> start;
		seArray<bool, 5> single;
		pattern(const std::string pnameOfFile) : nameOfFile(pnameOfFile), missionMis(), LineIndex(), uniqueid(), start(), single()	{
			loadPattern();
		}
		const std::string getFileName() const	{
			return nameOfFile;
		}
		bool ifAffiliate(const std::string puniqueid);
		void loadPattern();
};



class airGroupSection	{
	public:
		int AGSStart;
		seArray<std::string, 20> ags;
		airGroupSection(const int pAGSStart) : AGSStart(pAGSStart), ags()	{
		}
};



class weatherSection	{
	public:
		int weatherStart;
		seArray<std::string, 10> ws;
		weatherSection(const int pweatherStart) : weatherStart(pweatherStart), ws()	{
		}
};



class triangularPattern	{
		position tpp[3];
	public:
		triangularPattern() : tpp()	{
		}
		void setPattern(const double pperiod, const position ppos, const double pspeed);
		const position getPattern(int index) const	{
			return tpp[index];
		}
};



class squarePattern	{
		position spp[6];
	public:
		squarePattern() : spp()	{
		}
		void setPattern(const double pperiod, const position ppos, const double pspeed);
		const position getPattern(int index) const	{
			return spp[index];
		}
};



bool ifNotAttrited(const std::string puniqueid, mainTables& pmt, pattern& ppat);

const std::string getSingleLine(const std::string puniqueid, mainTables& pmt, pattern& ppat);


void copyPaths(mainTables& pmt, pattern& ppat);

void makeAGSection(mainTables& pmt, pattern& ppat, airGroupSection* pags);

std::string makeTime();

void save(mainTables& pasmt, pattern& paspat, modeSetter& pms);

position splitLine(const position& ppos1, const position& ppos2, const double distance);

double randAlt(double palt, double paltdiff);

void saveBriefings(mainTables& pmt, int pmissionNo);

int getRedFGroupNo(mainTables& pmt);						

int getBlueFGroupNo(mainTables& pmt);					

int getRedBGroupNo(mainTables& pmt);					

int getBlueBGroupNo(mainTables& pmt);					

int getRedJGroupNo(mainTables& pmt);					

int getBlueJGroupNo(mainTables& pmt);



class missionCount	{
		int missionNo;
	public:
		missionCount() : missionNo(0)	{
			load();
		}
		void load();
		void save();
		void set()	{
			++missionNo;
		}
		int get() const	{
			return missionNo;
		}
};



class timeFlow	{
		struct tm* currTime;
		long long startTime;
		bool before70;
		int prevMissionStart;
		int prevMissionDay;
		int prevMissionYear;
		int missionDuration;
	public:
		timeFlow() : startTime(0), missionDuration(0), before70(false), prevMissionStart(0), prevMissionDay(0)	{
			currTime = new struct tm;
			load();
		}
		void chkTime()	{
			if(currTime->tm_hour > 17 && currTime->tm_hour <= 24)	{
				currTime->tm_hour = 8;
				currTime->tm_min = 0;
				currTime->tm_mday = currTime->tm_mday + 1;
			}
			if(currTime->tm_hour < 8 && currTime->tm_hour >= 0)	{
				currTime->tm_hour = 8;
				currTime->tm_min = 0;
			}
			mktime(currTime);
		}
		void setTime(int pmin)	{
			currTime->tm_min = currTime->tm_min + pmin;
			mktime(currTime);
			if(currTime->tm_min < 30)
				currTime->tm_min = 0;
			if(currTime->tm_min >= 30)	{
				currTime->tm_hour = currTime->tm_hour + 1;
				currTime->tm_min = 0;
			}
			chkTime();
		}
		void assigntm(struct tm* pl, struct tm* pr)	{
			pl->tm_year = pr->tm_year;
			pl->tm_yday = pr->tm_yday;
			pl->tm_wday = pr->tm_wday;
			pl->tm_sec = pr->tm_sec;
			pl->tm_mon = pr->tm_mon;
			pl->tm_min = pr->tm_min;
			pl->tm_mday = pr->tm_mday;
			pl->tm_isdst = pr->tm_isdst;
			pl->tm_hour = pr->tm_hour;
		}
		const std::string getTime()	{
			struct tm* rt = new struct tm;
			assigntm(rt, currTime);
			if(before70)
				rt->tm_year = rt->tm_year - 52;
			return asctime(rt);
		}
		long long getLongTime()	{
			return mktime(currTime);
		}
		int getMissionTime() const	{
			return currTime->tm_hour;
		}
		void setMissionDuration();
		int getMissionDuration() const	{
			return missionDuration;
		}
		int getCampaignDuration()	{
			time_t start = startTime;
			time_t end = mktime(currTime);
			return difftime(end, start) / (3600*24);
		}
		void setPrevMissStart()	{
			prevMissionStart = currTime->tm_hour;
			prevMissionDay = currTime->tm_yday;
			prevMissionYear = currTime->tm_year;
		}
		int getTimeBetweenMissions()	{
			if((currTime->tm_hour + missionDuration / 60) > 16)
				return 120;
			if(currTime->tm_hour == 8)
				return 0;
			if((currTime->tm_year - prevMissionYear) != 0)
				prevMissionDay = 0;
			if((currTime->tm_yday - prevMissionDay) > 1)
				return (currTime->tm_yday - prevMissionDay) * 24 * 60;
			return (currTime->tm_hour - prevMissionStart) * 60;
		}
		void load();
		void save();
		const std::string getSeason() const	{
			if(((currTime->tm_mon == 2) && (currTime->tm_mday >= 21)) || (currTime->tm_mon == 3) || (currTime->tm_mon == 4) || ((currTime->tm_mon == 5) && (currTime->tm_mday < 22)))
				return "spring";
			if(((currTime->tm_mon == 5) && (currTime->tm_mday >= 22)) || (currTime->tm_mon == 6) || (currTime->tm_mon == 7) || ((currTime->tm_mon == 8) && (currTime->tm_mday < 23)))
				return "summer";
			if(((currTime->tm_mon == 8) && (currTime->tm_mday >= 23)) || (currTime->tm_mon == 9) || (currTime->tm_mon == 10) || ((currTime->tm_mon == 11) && (currTime->tm_mday < 22)))
				return "autumn";
			if(((currTime->tm_mon == 11) && (currTime->tm_mday >= 22)) || (currTime->tm_mon == 0) || (currTime->tm_mon == 1) || ((currTime->tm_mon == 2) && (currTime->tm_mday < 21)))
				return "winter";
		}
};



void moveVehicles(int pminutes);

std::string makeMainBriefingLine(mainTables& pmt);

void runAway(pattern& ppat);

void reverseCourse(pattern& ppat);

void produce(pattern& ppat, timeFlow& ptf);

void transport(pattern& ppat, timeFlow& ptf);

void reinforce(pattern& ppat, timeFlow& ptf);

void recreateAG(pattern& ppat, timeFlow& ptf);

void recreateC(pattern& ppat, timeFlow& ptf);

void recreateS(pattern& ppat, timeFlow& ptf);

void attrit(timeFlow& ptf);

void rebuild(pattern& ppat, timeFlow& ptf);

void denyFlight(mainTables& pmt, pattern& ppat);

void markFront(mainTables& pmt, pattern& ppat, modeSetter& pms);



class pilotLog	{
		std::string name;
		double flightHours;
		int AtAKills;
		bool alive;
		std::string groupname;
		std::string rank[10];
		int currentRank;
		std::string countryid;
		std::string side;
		std::string briefingApendLine;
		std::string baseName;
		int flightNumber;
		int planeNumber;
	public:
		pilotLog();
		void askRank();
		void askName()	{
			std::string iname;
			std::cout << "\nPlease type the name of the pilot:\n";
			std::cin >> iname;
			name = iname;
		}
		std::string getGroupID(std::string pfilename);
		void load(std::string pfname);
		void save();
		bool checkBase(std::string pgroupname, planesData& ppdata, facilitiesData& pfacdata);
		bool checkPlanes(std::string pgroupname, planesData& ppdata);
		std::string findAirGroup(std::string pgroupname, planesData& ppdata, facilitiesData& pfacdata);
		void checkAirGroup(std::string mfilename);
		std::string getBALine() const	{
			return briefingApendLine;
		}
		const std::string MPLine();
		void setFromKillLog();
		bool getDeath() const	{
			return alive;
		}
		void setFHours(timeFlow& ptf)	{
			flightHours += ((double)ptf.getMissionDuration()) / 60;
		}
		double getFHours() const	{
			return flightHours;
		}
		int getAtAKills() const	{
			return AtAKills;
		}
		void promote();
		std::string getRank() const	{
			return rank[currentRank];
		}
		std::string getName() const	{
			return name;
		}
		int getRankLimit();
		void resetLog();
		std::string getAFName()	{
			return baseName;
		}
		int getFlightNo()	{
			return flightNumber;
		}
		int getPlaneNo()	{
			return planeNumber;
		}
		void resetBALine()	{
			briefingApendLine = "NoString";
		}
		std::string getGroupName()	{
			return groupname;
		}
		void setGNameFF(std::string pfnme)	{
			groupname = getGroupID(pfnme);
		}
		const std::string getSide() const	{
			return side;
		}
		void switchSide()	{
			if(side == "blue")
				side = "red";
			if(side == "red")
				side = "blue";
		}
};


class checkBlitz	{										
	bool isBlitz;
public:
	checkBlitz() : isBlitz(0)	{
		load();
	}
	void load();
	void save();
	const bool get() const	{
		return isBlitz;
	}
};



void planMissions(mainTables& pmt, pilotLog& pplog);

void backup();

void reset();

void assesStrength(mainTables& pmt, pattern& ppat, pilotLog& pplog);

void makeWeatherSection(weatherSection* pws);

std::string makeMapLine(pattern& ppat);

void cleanup();

const std::string tabs(std::string pname);

bool copyFromBackup(const std::string pFolderName);


#endif
